
-- Compare versions procedure
DELIMITER //
CREATE PROCEDURE `COMPARE_VERSIONS`(IN `version1` VARCHAR(256), IN `version2` VARCHAR(256), OUT `result` VARCHAR(256) CHARSET utf8mb4 COLLATE utf8mb4_unicode_ci)
BEGIN
DECLARE i INT DEFAULT 0;
DECLARE j INT DEFAULT 0;
DECLARE nr1 INT DEFAULT 0;
DECLARE nr2 INT DEFAULT 0;
DECLARE comp INT DEFAULT 0;

  IF ISNULL(version1) THEN
    SET comp = -1;
  ELSEIF ISNULL(version2) THEN
    SET comp = 1;
  ELSE
  

      SET @l1 := (LENGTH(version1) - LENGTH(REPLACE(version1, '.', '')) + 1);
        SET @l2 := (LENGTH(version2) - LENGTH(REPLACE(version2, '.', '')) + 1);


        SET j = 1;
        SET @min := (SELECT IF(@l1 < @l2, @l1, @l2)); 
        SET comp = 0;

        WHILE (j<=@min AND comp=0) DO 


            SET @v1 := (SELECT REPLACE(SUBSTRING(SUBSTRING_INDEX(version1, '.', j), LENGTH(SUBSTRING_INDEX(version1, '.', j -1)) + 1), '.', ''));
            SET @v2 := (SELECT REPLACE(SUBSTRING(SUBSTRING_INDEX(version2, '.', j), LENGTH(SUBSTRING_INDEX(version2, '.', j -1)) + 1), '.', ''));

            SET @isNumber1 := (SELECT @v1 REGEXP '^[0-9]+$');
            SET @isNumber2 := (SELECT @v2 REGEXP '^[0-9]+$');

            IF (@isNumber1=1 AND @isNumber2=1) THEN

                SET nr1 := (SELECT 0 + @v1);
                SET nr2 := (SELECT 0 + @v2);
                IF nr1 < nr2 THEN
                    SET comp := -1;
                ELSEIF nr1>nr2 THEN
                    SET comp := 1;
                ELSE
                    SET comp := 0;
                END IF;

            ELSEIF @isNumber1=1 THEN
                SET comp := 1;

            ELSEIF @isNumber2=1 THEN 
                SET comp := -1;
            ELSE
                SET comp := (SELECT STRCMP(@v1, @v2));
            END IF;

            SET j := j + 1;
        END WHILE;


        IF (comp=0 AND @l1<>@l2) THEN

            IF @l1=@min THEN

                SET @v2 := (SELECT REPLACE(SUBSTRING(SUBSTRING_INDEX(version2, '.', j), LENGTH(SUBSTRING_INDEX(version2, '.', j -1)) + 1), '.', ''));
                SET @isNumber2 := (SELECT @v2 REGEXP '^[0-9]+$');
                SELECT @isNumber2;

                IF @isNumber2=1 THEN
                    SET comp := -1;
                ELSE
                    SET comp := 1;
                END IF;

            ELSE

                SET @v1 := (SELECT REPLACE(SUBSTRING(SUBSTRING_INDEX(version1, '.', j), LENGTH(SUBSTRING_INDEX(version1, '.', j -1)) + 1), '.', ''));
                SET @isNumber1 := (SELECT @v1 REGEXP '^[0-9]+$');
                IF @isNumber1=1 THEN
                    SET comp := 1;
                ELSE
                    SET comp := -1;
                END IF;
            END IF;

        END IF;
	END IF;
	SET result = comp;

End//
DELIMITER ;

-- New version procedure
DELIMITER //
CREATE PROCEDURE `COMPUTE_NEW_VERSION`(IN `versionParam` VARCHAR(256) CHARSET utf8mb4 COLLATE utf8mb4_unicode_ci, IN `product` VARCHAR(256)  CHARSET utf8mb4 COLLATE utf8mb4_unicode_ci, OUT `result` VARCHAR(256) CHARSET utf8mb4 COLLATE utf8mb4_unicode_ci)
BEGIN
DECLARE n INT DEFAULT 0;
DECLARE i INT DEFAULT 0;
DECLARE j INT DEFAULT 0;
DECLARE nextVersion varchar(256);
DECLARE previousVersion varchar(256);

CREATE TEMPORARY TABLE IF NOT EXISTS temporary_versions(
	version varchar(128) NOT NULL
)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

 INSERT INTO temporary_versions SELECT version FROM products INNER JOIN deployments ON products.productId = deployments.productId WHERE products.product = product;


SELECT COUNT(*) FROM temporary_versions INTO n;
SET i = 0;
WHILE i<n DO 
  SET @currentVer := (SELECT * FROM temporary_versions LIMIT i,1);
  
  CALL COMPARE_VERSIONS(versionParam, @currentVer, @res);
  IF @res = 1 THEN
	 -- A possible previous value, compare with the existing previous one and select the max value

	CALL COMPARE_VERSIONS(previousVersion, @currentVer, @prevRes);
	-- If current version is greater, set the new previousVersion
	-- Nothing to do on ELSE because the current previous value is the max value between the two values
	IF @prevRes=-1 THEN
		SET previousVersion := @currentVer;
	END IF;

	ELSE
		-- A possible next value, compare with the existing next one and select the min value
		CALL COMPARE_VERSIONS(nextVersion, @currentVer, @nextRes);
		-- If current version is smaller, set the new nextVersion
		-- Nothing to do on ELSE because the current next value is the min value between the two values
	IF (@nextRes=1 OR ISNULL(nextVersion)) THEN
		SET nextVersion := @currentVer;
	END IF;
  END IF;	

  SET i := i + 1;
END WHILE;

	IF ISNULL(nextVersion) THEN
		SET result := previousVersion;
	ELSE
		SET result := nextVersion;
	END IF;
	
End//
DELIMITER ;

-- Update deployment id
DELIMITER //
CREATE PROCEDURE `UPDATE_COMMENT_DEPLOYMENT_ID`()
BEGIN
DECLARE n INT DEFAULT 0;
DECLARE i INT DEFAULT 0;
  
	CREATE TEMPORARY TABLE IF NOT EXISTS deploys(
		product varchar(256) NOT NULL,
		version varchar(128) NOT NULL
	)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;
	
	INSERT INTO deploys SELECT DISTINCT product, version FROM comments WHERE deploymentId=0;
	
	SELECT COUNT(*) FROM deploys INTO n;
	SET i = 0;
	WHILE i<n DO 
    	SET @product := (SELECT product FROM deploys LIMIT i,1);
		SET @version := (SELECT version FROM deploys LIMIT i,1);
    	
		CALL COMPUTE_NEW_VERSION(@version, @product, @newVersion);
		
		SET @productId := (SELECT productId FROM products WHERE product=@product);

        SET @deploymentId := (SELECT deploymentId FROM deployments WHERE productId=@productId AND version=@newVersion);

		IF @deploymentId IS NOT NULL THEN
			UPDATE comments SET deploymentId=@deploymentId WHERE product=@product AND version=@version AND deploymentId=0 AND @deploymentId IS NOT NULL;
 		END IF;
 		
		SET i = i + 1;
	END WHILE;
	
	DROP TABLE IF EXISTS deploys;
	
END//
DELIMITER ;
 

 -- add deploymentId column
  ALTER TABLE `comments` ADD `deploymentId` int;
  
  -- remove the attribute "on update CURRENT_TIMESTAMP" from the date column when populating the comments table
  ALTER TABLE `comments` CHANGE `date` `date` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP;

  -- update comments.deploymentId value with corresponding deployments.deploymentId 
  UPDATE `comments` INNER JOIN `products` ON `comments`.`product`=`products`.`product` 
  INNER JOIN `deployments` ON `deployments`.`productId`=`products`.`productId` 
  SET `comments`.`deploymentId`=`deployments`.`deploymentId` 
  WHERE `comments`.`version`=`deployments`.`version` 
  AND `comments`.`product`=`products`.`product`; 
  
  UPDATE `comments` SET `deploymentId`=0 WHERE ISNULL(`deploymentId`);
  
  ALTER TABLE `comments` MODIFY COLUMN `deploymentId` int NOT NULL; 
    
  CALL UPDATE_COMMENT_DEPLOYMENT_ID();

  DELETE FROM `comments` WHERE `deploymentId`=0;
  
  -- add foreign key constraint for comments.deploymentId
  ALTER TABLE `comments`
    ADD CONSTRAINT `deploymentId` FOREIGN KEY (`deploymentId`) REFERENCES `deployments` (`deploymentId`);
  
  -- remove not null constraint for referedComment column
  ALTER TABLE `comments` MODIFY COLUMN `referedComment` int NULL; 
  
  -- set NULL instead of 0 for referedComment
  UPDATE `comments` SET `referedComment`=NULL WHERE `referedComment`=0;
  
  -- add foreign key constraint for comments.referedComment
  ALTER TABLE `comments`
    ADD CONSTRAINT `referedComment` FOREIGN KEY (`referedComment`) REFERENCES `comments` (`commentId`);

  -- set the anonymus user for the comments that no longet have the userId in the users table.
  UPDATE comments SET comments.userId = (SELECT userId FROM users WHERE email='anonymous@anonymous.com') WHERE NOT EXISTS (SELECT 1 FROM users WHERE comments.userId = users.userId);

  -- set back the attribute "on update CURRENT_TIMESTAMP" 
  ALTER TABLE `comments` CHANGE `date` `date` TIMESTAMP on update CURRENT_TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP;

  -- add foreign key constraint for comments.userId
  ALTER TABLE `comments`
    ADD CONSTRAINT `userId` FOREIGN KEY (`userId`) REFERENCES `users` (`userId`);
    
    -- drop version and product columns
  ALTER TABLE `comments`
	DROP COLUMN `product`,
	DROP COLUMN `version`;
	
DROP PROCEDURE IF EXISTS `UPDATE_COMMENT_DEPLOYMENT_ID`;
DROP PROCEDURE IF EXISTS `COMPUTE_NEW_VERSION`;
DROP PROCEDURE IF EXISTS `COMPARE_VERSIONS`;
