-- Set the organization ID for all existing page_views_record
UPDATE `page_views_record` SET `organizationId` = (SELECT sites.organizationId FROM sites WHERE page_views_record.siteId = sites.siteId);

-- #### user_subscriptions ####

-- Add not null constraint for organizationId column in user_subscriptions
ALTER TABLE user_subscriptions MODIFY organizationId INT NOT NULL;


-- Add foreign key constraint on the organizationId column.
ALTER TABLE `user_subscriptions` ADD FOREIGN KEY organization_fk_subscription(organizationId) REFERENCES organizations(id);

-- Remove foreign key and index on userId column
ALTER TABLE user_subscriptions
    DROP FOREIGN KEY user_subscriptions_ibfk_1,  
    DROP INDEX userId;
	
-- Remove column userId
ALTER TABLE user_subscriptions DROP COLUMN userId;

-- Rename user_subscriptions to organization_subscriptions
RENAME TABLE user_subscriptions TO organization_subscriptions;


-- #### user_subscription_notifications ####

-- Add not null constraint for organizationId column in user_subscriptions
ALTER TABLE user_subscription_notifications MODIFY organizationId INT NOT NULL;

-- Add foreign key constraint.
ALTER TABLE `user_subscription_notifications` ADD FOREIGN KEY organization_fk_subscr_notification(organizationId) REFERENCES organizations(id); 

-- Remove foreign key and index on userId column
ALTER TABLE user_subscription_notifications
    DROP FOREIGN KEY user_subscription_notifications_ibfk_1;

-- Drop the primary key
ALTER TABLE user_subscription_notifications DROP PRIMARY KEY;
	
-- Remove column userId
ALTER TABLE user_subscription_notifications DROP COLUMN userId;

-- Add the primary key as organization id and notification type.
ALTER TABLE user_subscription_notifications ADD PRIMARY KEY (organizationId, notificationType);

-- Rename user_subscriptions to organization_subscriptions
RENAME TABLE user_subscription_notifications TO organization_subscription_notifications;


-- #### sites ####

-- Add not null constraint for organizationId column in sites
ALTER TABLE sites MODIFY organizationId INT NOT NULL;

-- Add foreign key constraint.
ALTER TABLE `sites` ADD FOREIGN KEY organization_fk_sites(organizationId) REFERENCES organizations(id); 

-- #### subscription_history ####

-- Add not null constraint for subscriptionId
ALTER TABLE subscription_history MODIFY subscriptionId INT NOT NULL;

-- Add foreign key constraint on the subscriptionId column.
ALTER TABLE `subscription_history` ADD FOREIGN KEY subscription_fk(subscriptionId) REFERENCES organization_subscriptions(id);

-- Delete userId column from subscription_history because we linked subscription to subscription_history
ALTER TABLE subscription_history DROP FOREIGN KEY subscription_history_ibfk_1;
ALTER TABLE subscription_history DROP COLUMN userId;


-- #### organizations ####

-- Remove organizations.ownerId column 
ALTER TABLE organizations
    DROP FOREIGN KEY organizations_ibfk_1,  
    DROP INDEX ownerId;
    
ALTER TABLE organizations DROP COLUMN ownerId;


-- #### page_views_record ####

-- Add not null constraint for organizationId column in page_views_record
ALTER TABLE page_views_record MODIFY organizationId INT NOT NULL;

-- Add foreign key constraint on the organizationId column.
ALTER TABLE `page_views_record` ADD FOREIGN KEY organization_fk_view_records(organizationId) REFERENCES organizations(id);

-- Add index for organizationId
CREATE INDEX organization_index ON page_views_record(organizationId);

-- Remove ownerID column
ALTER TABLE page_views_record DROP INDEX owner_index;
ALTER TABLE page_views_record DROP COLUMN ownerId;


-- Delete user details for old users
DELETE FROM `user_details` WHERE user_details.userId IN (SELECT userId FROM users WHERE users.organizationId IS NULL);


--  Delete platform_credentials entries for old users
DELETE FROM `platform_credentials` WHERE platform_credentials.userId IN (SELECT userId FROM users WHERE users.organizationId IS NULL);


--  Delete oauth_credentials entries for old users
DELETE FROM `oauth_credentials` WHERE oauth_credentials.userId IN (SELECT userId FROM users WHERE users.organizationId IS NULL);

-- Delete entries for old users that are not in any organization
DELETE FROM `email_notifications` WHERE email_notifications.userId IN (SELECT userId FROM users WHERE users.organizationId IS NULL);

-- Remove all users with organizationId = null
DELETE FROM `users` WHERE organizationId IS NULL;

-- Add NOT NULL and Foreign key constraints
ALTER TABLE users MODIFY organizationId INT NOT NULL;
ALTER TABLE users MODIFY organizationRole varchar(256) NOT NULL;
ALTER TABLE users ADD FOREIGN KEY organization_fk_users(organizationId) REFERENCES organizations(id);
