/* Temporary table -> Links all comments to their top level ancestor comment (threadId) */
CREATE TEMPORARY TABLE temp_comments(
    commentId SERIAL PRIMARY KEY,
	threadId INT
);
/* 
  Recursive CTE that links the thread ID (top level ancestor comment ID) to each comment 
  and inserts all entries in the temporary table.
*/ 
WITH RECURSIVE threads AS (
	/* Anchor query -> top level comments */
	/* Store the ID of each top level comment as `threadId` */
    SELECT *, commentId AS threadId FROM comments WHERE referedComment IS NULL
    UNION ALL 
	/* Recursive Join -> Replies */
	/* Propagate the ID of the top level comment as `threadId` */
    SELECT c.*, t.threadId 
    FROM comments AS c INNER JOIN threads AS t
    ON c.referedComment = t.commentId
	WHERE c.referedComment IS NOT NULL  
)
INSERT INTO temp_comments(commentId, threadId) 
SELECT threads.commentId, threads.threadId FROM threads;

/* Move all replies on level 2 setting their topl level ancestor comment as parent comment. */
UPDATE comments
SET 
    referedComment = temp_comments.threadId
FROM 
    temp_comments
WHERE
    comments.commentId = temp_comments.commentId 
	AND comments.referedComment is not NULL;

/* Drop the temporary table. */
DROP TABLE temp_comments;