-- Set the organization ID for all existing page_views_record
UPDATE page_views_record SET organizationId = (SELECT sites.organizationId FROM sites WHERE page_views_record.siteId = sites.siteId);

------ user_subscriptions ----------

-- Add not null constraint for organizationId column in user_subscriptions
ALTER TABLE user_subscriptions ALTER COLUMN organizationId SET NOT NULL;


-- Add foreign key constraint on the organizationId column.
ALTER TABLE user_subscriptions ADD FOREIGN KEY (organizationId) REFERENCES organizations(id);

-- Remove foreign key and index on userId column
ALTER TABLE user_subscriptions DROP CONSTRAINT user_subscriptions_userId_fkey;
    
	
-- Remove column userId
ALTER TABLE user_subscriptions DROP COLUMN userId;

-- Rename user_subscriptions to organization_subscriptions
ALTER TABLE user_subscriptions RENAME TO organization_subscriptions;


------ user_subscription_notifications ----------

-- Add not null constraint for organizationId column in user_subscriptions
ALTER TABLE user_subscription_notifications ALTER COLUMN organizationId SET NOT NULL;

-- Add foreign key constraint.
ALTER TABLE user_subscription_notifications ADD FOREIGN KEY (organizationId) REFERENCES organizations(id);

-- Remove foreign key and index on userId column
ALTER TABLE user_subscription_notifications DROP CONSTRAINT user_subscription_notifications_userId_fkey;

-- Drop the primary key
ALTER TABLE user_subscription_notifications DROP CONSTRAINT user_subscription_notifications_pkey;
	
-- Remove column userId
ALTER TABLE user_subscription_notifications DROP COLUMN userId;

-- Add the primary key as organization id and notification type.
ALTER TABLE user_subscription_notifications ADD PRIMARY KEY (organizationId, notificationType);

-- Rename user_subscriptions to organization_subscriptions
ALTER TABLE user_subscription_notifications RENAME TO organization_subscription_notifications;


------- sites ------

-- Add not null constraint for organizationId column in sites
ALTER TABLE sites ALTER COLUMN organizationId SET NOT NULL;

-- Add foreign key constraint.
ALTER TABLE sites ADD FOREIGN KEY (organizationId) REFERENCES organizations(id); 

----------- subscription_history ----------

-- Add not null constraint for subscriptionId
ALTER TABLE subscription_history ALTER COLUMN subscriptionId SET NOT NULL;

-- Add foreign key constraint on the subscriptionId column.
ALTER TABLE subscription_history ADD FOREIGN KEY (subscriptionId) REFERENCES organization_subscriptions(id);

-- Delete userId column from subscription_history because we linked subscription to subscription_history
ALTER TABLE subscription_history DROP CONSTRAINT subscription_history_userId_fkey;
ALTER TABLE subscription_history DROP COLUMN userId;


-------- organizations ------------

-- Remove organizations.ownerId column 
ALTER TABLE organizations
    DROP CONSTRAINT organizations_ownerId_fkey;
    
ALTER TABLE organizations DROP COLUMN ownerId;


--------- page_views_record -----------------

-- Add not null constraint for organizationId column in page_views_record
ALTER TABLE page_views_record ALTER COLUMN organizationId SET NOT NULL;

-- Add foreign key constraint on the organizationId column.
ALTER TABLE page_views_record ADD FOREIGN KEY (organizationId) REFERENCES organizations(id);

-- Add index for organizationId
CREATE INDEX organization_index ON page_views_record(organizationId);

-- Remove ownerID column
DROP INDEX page_views_record_owner_index;
ALTER TABLE page_views_record DROP COLUMN ownerId;



-- Delete user details for old users
DELETE FROM user_details WHERE user_details.userId IN (SELECT userId FROM users WHERE users.organizationId IS NULL) AND user_details.role != 'ROLE_APPLICATION_ADMIN';


--  Delete platform_credentials entries for old users
DELETE FROM platform_credentials WHERE platform_credentials.userId IN (SELECT userId FROM users WHERE users.organizationId IS NULL) AND platform_credentials.userId NOT IN (SELECT userId FROM user_details WHERE user_details.role = 'ROLE_APPLICATION_ADMIN');


--  Delete oauth_credentials entries for old users
DELETE FROM oauth_credentials WHERE oauth_credentials.userId IN (SELECT userId FROM users WHERE users.organizationId IS NULL);

-- Delete entries for old users that are not in any organization
DELETE FROM email_notifications WHERE email_notifications.userId IN (SELECT userId FROM users WHERE users.organizationId IS NULL) AND email_notifications.userId NOT IN (SELECT userId FROM user_details WHERE user_details.role = 'ROLE_APPLICATION_ADMIN');

--- Delete users with organization NULL
DELETE FROM users WHERE organizationId IS NULL AND userId NOT IN (SELECT userId FROM user_details WHERE user_details.role = 'ROLE_APPLICATION_ADMIN');

-- Add NOT NULL and Foreign key constraints
ALTER TABLE users ALTER COLUMN organizationId SET NOT NULL;
ALTER TABLE users ALTER COLUMN organizationRole SET NOT NULL;
ALTER TABLE users ADD FOREIGN KEY (organizationId) REFERENCES organizations(id);

-- Add unique key for (email, organizationId) for users

ALTER TABLE users DROP CONSTRAINT users_email_key;

CREATE UNIQUE INDEX users_organizations_index ON users(email, organizationId);

ALTER TABLE users ADD CONSTRAINT users_organizations_key UNIQUE USING INDEX users_organizations_index;
